/*
  Copyright (c) 2013 Jonas Kuemmerlin <rgcjonas@gmail.com>

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the GNOME nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Convenience = Extension.imports.convenience;
const Meta = imports.gi.Meta;
const Main = imports.ui.main;

var settings;

var keybindingExpand;
var keybindingHide;

var restoreClose;
var restoreExpanded;

function init() {
    settings = Convenience.getSettings();
}

function enable() {
    keybindingExpand = global.display.add_keybinding('expand-notification', settings, Meta.KeyBindingFlags.NONE, expandNotification);
    
    //our 'hooks'
    if (typeof(Main.messageTray._closeNotification) != "undefined") { //GS 3.8
        restoreClose = injectIntoFunction(Main.messageTray, "_closeNotification", onClose);
    } else { //GS 3.6
        restoreClose = injectIntoFunction(Main.messageTray, "_onCloseClicked", onClose);
    }
    restoreExpanded = injectIntoFunction(Main.messageTray, "_onNotificationExpanded", onExpand);
}

function disable() {
    global.display.remove_keybinding(keybindingExpand);
    
    restoreClose();
    restoreExpanded();
}

function onExpand() {
    keybindingHide = this._notification.actor.connect('key-press-event', function(actor, event) {
        if (event.get_key_code() == settings.get_int('hide-notification')) {
            closeNotification();
        }
    });
}

function onClose() {
    this._notification.actor.disconnect(keybindingHide);
}

function expandNotification() {
    if (Main.messageTray._notification) {
        Main.messageTray._expandNotification(false);
    }
}

function closeNotification() {
    if (typeof(Main.messageTray._closeNotification) != "undefined") { //GS 3.8
        Main.messageTray._closeNotification();
    } else { //GS 3.6
        Main.messageTray._onCloseClicked();
    }
}

//injects a function call before the original function, returns a function which can be used to restore the original one
function injectIntoFunction(object, funcName, our) {
    var old = object[funcName];
    object[funcName] = function() {
        our.apply(this, arguments);
        old.apply(this, arguments);
    }
    return function() {
        object[funcName] = old;
    }
}